@@ -107,6 +107,7 @@ struct ftp_supported_feature ftp_supported_features[] =
    { FTP_FEAT_PRET, "PRET" },
    { FTP_FEAT_EPSV, "EPSV" }, /* rfc2428 */
    { FTP_FEAT_EPRT, "EPRT" }, /* rfc2428 */
+   { -1, NULL },
 };
 
 static void rfc2389_feat_supported_cmd (ftp_protocol_data * ftpdat, char * cmd)
@@ -115,7 +116,7 @@ static void rfc2389_feat_supported_cmd (ftp_protocol_data * ftpdat, char * cmd)
    char *p = cmd;
    int i;
    while (*p <= 32) p++; // strip leading spaces
-   for (i = 0; i < FTP_FEAT_TOTAL; i++)
+   for (i = 0; ftp_supported_features[i].str; i++)
    {
       if (strcmp (p, ftp_supported_features[i].str) == 0) {
           ftpdat->feat[i] = 1;
@@ -137,7 +138,13 @@ static void rfc2389_feat_supported_cmd (ftp_protocol_data * ftpdat, char * cmd)
 static int ftp_read_response (gftp_request * request, int disconnect_on_42x)
 {
   //DEBUG_PRINT_FUNC
-  char tempstr[512], code[4];
+  // The retrieved string from the socket may contain 1 or more lines, 1 or more numeric codes
+  //   but the function must return the first numeric code
+  // The buffer should retain the rest of the string to be processed
+  //   in the next call to this function
+  // Only after processing the stored lines, the function must retrieve
+  //   more data from the socket
+  char code[4];
   ftp_protocol_data * ftpdat;
   ssize_t ret;
   int lines = 0;
@@ -147,69 +154,82 @@ static int ftp_read_response (gftp_request * request, int disconnect_on_42x)
   char * last_line = NULL;
   char * pos = NULL;
   int line_ending = 0;
+  size_t buffer_size = 2048; //sizeof(ftpdat->response_buffer);
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  if (request->last_ftp_response)
-    {
+  if (request->last_ftp_response) {
       g_free (request->last_ftp_response);
       request->last_ftp_response = NULL;
-    }
+  }
 
   ftpdat = request->protocol_data;
 
+  if (ftpdat->response_buffer == NULL) {
+     ftpdat->response_buffer = malloc (buffer_size);
+     ftpdat->response_buffer[0] = 0;
+  }
+  pos = ftpdat->response_buffer_pos;
+
+  ftpdat->last_response_code = 0;
   *code = '\0';
-  *tempstr = 0;
   *fixed_line = 0;
+
   while (1)
   {
-     line = str_get_next_line_pointer (tempstr, &pos, &line_ending);
+     line = str_get_next_line_pointer (ftpdat->response_buffer, &pos, &line_ending);
      if (!line)
      {
         // read text chunk, may not contain the whole response
         //DEBUG_TRACE("$$$  request->read_function: %p\n", request->read_function)
-        ret = request->read_function (request, tempstr, sizeof(tempstr), request->datafd);
-        if (ret <= 0) {
+        ret = request->read_function (request, ftpdat->response_buffer, buffer_size, request->datafd);
+        if (ret <= 0) { /* reset buffer */
+           ftpdat->response_buffer_pos = NULL;
+           ftpdat->response_buffer[0] = 0;
            break;
         }
-        tempstr[ret] = '\0';
-        // reset pos & lines
+        ftpdat->response_buffer[ret] = '\0';
+        /* reset pos & lines */
         pos = NULL;
         lines = 0;
-        // count lines
-        for (p = tempstr; *p; p++) {
+        /* count lines */
+        for (p = ftpdat->response_buffer; *p; p++) {
            if (*p == '\n') lines++;
         }
-        if (!lines) {
+        if (!lines) { /* reset buffer */
+           ftpdat->response_buffer_pos = NULL;
+           ftpdat->response_buffer[0] = 0;
            break;
         }
-        //DEBUG_TRACE(">>> %s <<<\n",tempstr)
+        //DEBUG_TRACE("%d##%d>>> %s <<<\n",lines, strlen(ftpdat->response_buffer), ftpdat->response_buffer)
         continue;
      }
-
+     ftpdat->response_buffer_pos = pos;
+/*
      if (pos)
      {
         if (!line_ending) {
            // incomplete line, will need to fix
            snprintf (fixed_line, sizeof(fixed_line), "%s", line);
-           puts ("a");
+           DEBUG_PUTS ("@ incomplete line, will need to fix\n")
            continue;
         } else if (*fixed_line) {
            // copy and complete line
            snprintf (fixed_line + strlen(fixed_line), sizeof(fixed_line) - strlen(fixed_line) - 1,
                      "%s", line);
            line = fixed_line;
-           puts ("b");
+           DEBUG_PUTS ("@@ copy and complete line\n")
         }
      }
-
-     last_line = line;
-
+*/
      if (isdigit((int)line[0]) && isdigit((int)line[1]) && isdigit((int)line[2]))
-     { // reply code has been identified, it will be used to determine when to stop
-         strncpy (code, line, 3);
-         code[3] = ' ';
+     {
+        if (!*code) {
+           // reply code has been identified, it will be used to determine when to stop
+           strncpy (code, line, 3);
+           code[3] = ' ';
+        }
      }
 
      if (ftpdat->last_cmd == FTP_CMD_FEAT) {
@@ -220,12 +240,19 @@ static int ftp_read_response (gftp_request * request, int disconnect_on_42x)
         else
            request->logging_function (gftp_logging_recv, request, "%s\n", line);
      }
+
      if (*code && strncmp (code, line, 4) == 0) {
-         break;
+        code[3] = '\0';
+        ftpdat->last_response_code = atoi (code);
+        code[3] = ' ';
+        last_line = line;
+        break; /* 'XXX ' end of current response */
      }
+/*
      if (line_ending && *fixed_line) {
         *fixed_line = 0;
      }
+*/
   }
 
   if (!last_line) {
@@ -688,6 +715,8 @@ int ftp_connect (gftp_request * request)
     }
 
   // determine server features, the response to this is not logged
+  ftpdat->feat[FTP_FEAT_LIST_AL] = 1; /* use LIST -al by default */
+  ftpdat->feat[FTP_FEAT_SITE] = 1;
   ftpdat->last_cmd = FTP_CMD_FEAT;
   ret = ftp_send_command (request, "FEAT\r\n", -1, 1, 0);
   ftpdat->last_cmd = 0;
@@ -815,6 +844,14 @@ static int ftp_do_data_connection_new (gftp_request * request)
   {
       if (USE_EPSV) {
           resp = ftp_send_command (request, "EPSV\r\n", -1, 1, 1);
+          if (ftpdat->last_response_code == 500) { // unrecognized command
+              // - deal with a broken server or pure-ftpd -b
+              // - this error is only possible if using ipv4
+              USE_EPSV = USE_EPRT = 0;
+              ftpdat->feat[FTP_FEAT_EPSV] = 0;
+              ftpdat->feat[FTP_FEAT_EPRT] = 0;
+              resp = ftp_send_command (request, "PASV\r\n", -1, 1, 1);
+          }
       } else {
           resp = ftp_send_command (request, "PASV\r\n", -1, 1, 1);
       }
@@ -1139,8 +1176,12 @@ static off_t ftp_get_file (gftp_request * request, const char *filename,
   ftpdat = request->protocol_data;
   if (passive_transfer && ftpdat->feat[FTP_FEAT_PRET]) {
     ret = ftp_generate_and_send_command (request, "PRET RETR", filename,1, 0);
-    if (ret < 0)
+    if (ftpdat->last_response_code == 500) {
+       ftpdat->feat[FTP_FEAT_PRET] = 0; // disable silently
+    }
+    if (ret < 0) {
        return (ret);
+    }
   }
 
   ret = ftp_setup_file_transfer (request, filename, startsize, "RETR");
@@ -1180,6 +1221,9 @@ static int ftp_put_file (gftp_request * request, const char *filename,
   ftpdat = request->protocol_data;
   if (passive_transfer && ftpdat->feat[FTP_FEAT_PRET]) {
     ret = ftp_generate_and_send_command (request, "PRET STOR", filename,1, 0);
+    if (ftpdat->last_response_code == 500) {
+       ftpdat->feat[FTP_FEAT_PRET] = 0; // disable silently
+    }
   }
 
   if (ftpdat->data_connection < 0 && 
@@ -1194,6 +1238,7 @@ static off_t ftp_transfer_file (gftp_request *fromreq, const char *fromfile,
                                 off_t fromsize, gftp_request *toreq, 
                                 const char *tofile, off_t tosize)
 {
+  // FXP, transfer between FTP servers. Untested
   DEBUG_PRINT_FUNC
   char *tempstr, *pos, *endpos;
   ftp_protocol_data * ftpdat, * ftpfrom, * ftpto;
@@ -1278,7 +1323,6 @@ static int ftp_end_transfer (gftp_request * request)
   ftp_close_data_connection (request);
 
   ret = ftp_read_response (request, 1);
-
   if (ret < 0)
     return (ret);
   else if (ret == '2')
@@ -1328,11 +1372,17 @@ static int ftp_list_files (gftp_request * request)
 
   if (ftpdat->feat[FTP_FEAT_MLSD]) {
      ret = ftp_send_command (request, "MLSD\r\n", -1, 1, 0);
-  } else {
+     if (ftpdat->last_response_code == 500) {
+         ftpdat->feat[FTP_FEAT_MLSD] = 0;
+         ret = ftp_send_command (request, "LIST\r\n", -1, 1, 0);
+     }
+  } else if (ftpdat->feat[FTP_FEAT_LIST_AL]) {
      ret = ftp_send_command (request, "LIST -al\r\n", -1, 1, 0);
-  }
-  // fall back to LIST if an error response is sent..
-  if (ret > 0 && ret != '1') {
+     if (ftpdat->last_response_code == 550) {
+        ftpdat->feat[FTP_FEAT_LIST_AL] = 0;
+        ret = ftp_send_command (request, "LIST\r\n", -1, 1, 0);
+     }
+  } else {
      ret = ftp_send_command (request, "LIST\r\n", -1, 1, 0);
   }
 
@@ -1516,8 +1566,17 @@ int ftp_get_next_file (gftp_request * request, gftp_file * fle, int fd)
           if (strncmp (tempstr, "total", strlen ("total")) != 0 &&
               strncmp (tempstr, _("total"), strlen (_("total"))) != 0)
           {
-            request->logging_function (gftp_logging_error, request,
+            if (strstr(tempstr,"-al") && (strstr(tempstr,"No such") || strstr(tempstr,"no such")))
+            {
+                // TODO: trigger a LIST again
+                ftpdat->feat[FTP_FEAT_LIST_AL] = 0;
+                request->logging_function (gftp_logging_error, request,
+                         "It looks like LIST -al is not supported\n");
+
+            } else {
+                request->logging_function (gftp_logging_error, request,
                      _("Warning: Cannot parse listing %s\n"), tempstr);
+            }
           }
             gftp_file_destroy (fle, 0);
             continue;
@@ -1654,11 +1713,18 @@ static int ftp_chmod (gftp_request * request, const char *file, mode_t mode)
   char *tempstr, *utf8;
   size_t destlen;
   int ret;
+  ftp_protocol_data * ftpdat;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (file != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
+  ftpdat = request->protocol_data;
+  if (!ftpdat->feat[FTP_FEAT_SITE]) {
+     request->logging_function (gftp_logging_error, request,"The server doesn't support SITE CHMOD\n");
+     return (GFTP_ERETRYABLE);
+  }
+
   utf8 = gftp_filename_from_utf8 (request, file, &destlen);
   if (utf8 != NULL)
     {
@@ -1669,6 +1735,9 @@ static int ftp_chmod (gftp_request * request, const char *file, mode_t mode)
     tempstr = g_strdup_printf ("SITE CHMOD %o %s\r\n", mode, file);
 
   ret = ftp_send_command (request, tempstr, -1, 1, 0);
+  if (ftpdat->last_response_code) {
+      ftpdat->feat[FTP_FEAT_SITE] = 0;
+  }
   g_free (tempstr);
 
   if (ret < 0)
@@ -1686,11 +1755,18 @@ static int ftp_site (gftp_request * request, int specify_site, const char *comma
   char *tempstr, *utf8;
   size_t len;
   int ret;
+  ftp_protocol_data * ftpdat;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (command != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
+  ftpdat = request->protocol_data;
+  if (!ftpdat->feat[FTP_FEAT_SITE]) {
+     request->logging_function (gftp_logging_error, request,"The server doesn't support the SITE command\n");
+     return (GFTP_ERETRYABLE);
+  }
+
   utf8 = gftp_string_from_utf8 (request, -1, command, &len);
   if (utf8 != NULL)
     {
@@ -1722,6 +1798,9 @@ static int ftp_site (gftp_request * request, int specify_site, const char *comma
     }
 
   ret = ftp_send_command (request, tempstr, len, 1, 0);
+  if (ftpdat->last_response_code) {
+      ftpdat->feat[FTP_FEAT_SITE] = 0;
+  }
   g_free (tempstr);
 
   if (ret < 0)
@@ -1756,6 +1835,11 @@ static void ftp_request_destroy (gftp_request * request)
 
   ftpdat = request->protocol_data;
 
+  if (ftpdat->response_buffer) {
+     free (ftpdat->response_buffer);
+     ftpdat->response_buffer = NULL;
+     ftpdat->response_buffer_pos = NULL;
+  }
   if (ftpdat->dataconn_rbuf != NULL)
     gftp_free_getline_buffer (&ftpdat->dataconn_rbuf);
 }
@@ -1780,7 +1864,7 @@ static void ftp_copy_param_options (gftp_request * dest_request,
   dftpdat->data_conn_tls_close = sftpdat->data_conn_tls_close;
   dftpdat->list_type           = sftpdat->list_type;
   dftpdat->last_cmd            = sftpdat->last_cmd;
-  memcpy (dftpdat, sftpdat, sizeof(*sftpdat));
+  memcpy (dftpdat->feat, sftpdat->feat, sizeof(sftpdat->feat));
 
   dest_request->read_function = src_request->read_function;
   dest_request->write_function = src_request->write_function;
